dofile("sys/lua/statistics/admin_list.lua")
dofile("sys/lua/statistics/gajos_lib.lua")
dofile("sys/lua/statistics/usgnname/GetUSGNName.lua")

stats = { users = {} }
stats.user = lib.initArray(32, {})

function stats.initUser(usgn)
	if usgn > 0 and not stats.users[usgn] then
		stats.users[usgn] = {
			playTime = 0;
			weekPlayTime = {};
		}
		
		for i = 1, 7 do
			stats.users[usgn].weekPlayTime[i] = 0
		end
	end
end

function stats.loadUsers()
	local file = io.open("sys/lua/statistics/saves/statistics.dat", "r")
	
	if not file then return end
	
	for line in file:lines() do
		local tab = lib.toTable(line)
		local usgn = tonumber(tab[1])
		local playTime = tonumber(tab[2])
		local weekPlayTime = {}
		
		for i = 1, 7 do
			weekPlayTime[i] = tonumber(tab[i + 2])
		end
		
		stats.initUser(usgn)
		stats.users[usgn].playTime = playTime
		stats.users[usgn].weekPlayTime = weekPlayTime
	end
	
	file:close()
end

function stats.saveUsers()
	local file = io.open("sys/lua/statistics/saves/statistics.dat", "w")
	
	for usgn in pairs(stats.users) do
		local u = stats.users[usgn]
		local str = ""
		
		for i = 1, 7 do
			str = str .. u.weekPlayTime[i] .. " "
		end
		
		file:write(usgn .. " " .. u.playTime .. " " .. str .. "\n")
	end
	
	file:close()
end

function stats.draw(id, usgn)
	if stats.user[id].hudid then return end
	
	if usgn > 0 and stats.users[usgn] then
		if not stats.user[id].hudid then
			local function getMaxTime()
				local max = 0
				for i = 1, 7 do
					local _time = stats.users[usgn].weekPlayTime[i]
					if _time > max then
						max = _time
					end
				end
				return max
			end
			local dayOfWeek = lib.getDayOfWeek(os.date("%d"), os.date("%m"), os.date("%y"))
			
			stats.user[id].hudid = image("gfx/stats/hud.png", 320, 240, 2, id)
			stats.user[id].weekid = {}
			
			local xSize = 0.8
			local ySize = 2.6
			
			hudtxt2(id, 18, "\169255255255" .. getUSGNName(usgn) .. " - [minutes]", 194, 142, 0)
			local playTime = stats.users[usgn].playTime
			hudtxt2(id, 19, "\169255255255Play time: " .. playTime .. " min @ " .. lib.round(playTime / 60, 2) .. " h", 200, 303, 0)
			
			for i = 1, 7 do	
				local x = 214 + (i - 1) * 35
				local y = 230
				
				stats.user[id].weekid[i] = image("gfx/block.bmp", x, y, 2, id)
				
				local weekPlayTime = stats.users[usgn].weekPlayTime[i]
				local days = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"}
				hudtxt2(id, 10 + i, "\169" .. (dayOfWeek[2] == days[i] and "255000000" or "255255255") .. days[i], x, 273, 1)
				hudtxt2(id, 20 + i, "\169" .. (dayOfWeek[2] == days[i] and "255000000" or "255255255") .. weekPlayTime .. "m", x, 171, 1)
				
				local img = stats.user[id].weekid[i]
				local yScale = weekPlayTime / getMaxTime()
				imagescale(img, xSize, ySize * (yScale == 0 and 0.01 or yScale))
				imagepos(img, x, y + 41.6 * (1 - yScale), 0) -- 41.6 = 32 * ySize / 2
				
				if i == dayOfWeek[1] then
					imagecolor(img, 190, 0, 0)
				else
					imagecolor(img, 200, 200, 200)
				end
			end
		end
	end
end

function stats.remove(id)
	if not stats.user[id].hudid then return end
	
	freeimage(stats.user[id].hudid)
	stats.user[id].hudid = nil
	
	for i = 1, 7 do
		freeimage(stats.user[id].weekid[i])
		hudtxt2(id, 10 + i, " ", 0, 0, 0)
		hudtxt2(id, 20 + i, " ", 0, 0, 0)
	end
	
	hudtxt2(id, 18, " ", 0, 0, 0)
	hudtxt2(id, 19, " ", 0, 0, 0)
end

function stats.updatePerMinute()
	for _, id in pairs(player(0, "table")) do
		local usgn = player(id, "usgn")
		if stats.users[usgn] then
			local playTime = stats.users[usgn].playTime
			local weekPlayTime = stats.users[usgn].weekPlayTime
			local dayOfWeek = lib.getDayOfWeek(os.date("%d"), os.date("%m"), os.date("%y"))[1]
			
			stats.users[usgn].playTime = playTime + 1
			stats.users[usgn].weekPlayTime[dayOfWeek] = weekPlayTime[dayOfWeek] + 1
		end
	end
end

function getUSGNName(usgn)
	local file = io.open('sys/lua/statistics/usgnname/USGNNames/' .. usgn, 'r')
	local name = 'Unknown'
	if file then
		name = file:read():sub(11)
		file:close()
	end
	return name
end

stats.loadUsers()

-- HOOKS --
function stats.onSay(id, txt)
	local usgn = player(id, "usgn")
	local cmd = lib.toTable(txt)
	
	local isStaff = function(id)
		for _, i in pairs(STATS_admin_list) do
			if i == usgn then
				return true
			end
		end
		return false
	end
	
	serverMessage = function(id, txt)
		local msgColor = '050150255'
		local txtColor = '255255255'
		if id > 0 then
			msg2(id, '\169' .. msgColor .. '[SERVER]: \169' .. txtColor .. txt)
		else
			msg('\169' .. msgColor .. '[SERVER]: \169' .. txtColor .. txt)
		end
	end
	
	local noper = function()
		serverMessage(id, 'You do NOT have permissions to do that')
	end
	
	local noe = function()
		serverMessage(id, 'This player does NOT exist')
	end
	
	if cmd[1] == "!stats" then
		if isStaff(id) then
			if not stats.user[id].hudid then
				local pl = tonumber(cmd[2])
				
				if pl then
					if player(pl, 'exists') and not player(pl, 'bot') then
						stats.draw(id, player(pl, 'usgn'))
							serverMessage(id, 'Type !stats again to hide the HUD.')
					elseif pl > 32 then
						if stats.users[pl] then
							stats.draw(id, pl)
							serverMessage(id, 'Type !stats again to hide the HUD.')
						else
							noe()
						end
					else
						noe()
					end
				else
					serverMessage(id, '!stats <id/usgn>')
				end
			else
				stats.remove(id)
			end
		else
			noper()
		end
		
		return 1
	end
end

function stats.onJoin(id)
	local usgn = player(id, "usgn")
	
	stats.initUser(usgn)
	stats.remove(id)
end

function stats.onLeave(id)
	stats.remove(id)
end

function stats.onMinute()
	stats.updatePerMinute()
	stats.saveUsers()
end

addhook("say", "stats.onSay")
addhook("join", "stats.onJoin")
addhook("leave", "stats.onLeave")
addhook("minute", "stats.onMinute")